/*
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
 *  Author: Andreas Brauchli <andreas.brauchli@aizo.com>
 *  Author: Roman K�hler <roman.koehler@aizo.com>
 */



Ext.String.formatRe2=/\{([A-Za-z0-9_#]+)\}/g;
Ext.String.format2= function(format,args) {
    return format.replace(Ext.String.formatRe2, function(m, i) {
    	if (args==null)
    		return "";
        if (args[i]!=null)  {
        	if (typeof(_) != "undefined")
        		return _(args[i]);
        	else
        		return (args[i]);
        }
        else
            return "";
    });
};


/*
 * moc: id 1 Brightness :  	0 - 100  %
 *         2 Hue		:	0 - 358,6  degree
 *         3 Saturation : 	0 - 100    %
 *         4 colortemp	:	100 - 1000  mired
 *         5 x			:	0  - 1.0 cie color model
 *         6 y			:	0  - 1.0 cie color model
 *         7 Position (vertical) 		: 0 - 100    %
 *         8 Position (horizontal)		: 0 - 100    %
 *         9 Position (opening angle)	: 0 - 100    %
 *         10 Permeablility	: 0 - 100    %
 * */

/* Converts an array of three R/G/B values to a string containing an HTML hexadecimal color value.
*
* @param rgb array of the three R/G/B values, in the 0..255 range
* @return HTML color value (format: #RRGGBB) */
function rgb2hex(rgb)
{
	// slightly modified version of http://stackoverflow.com/a/19765382
	var rgb = rgb[2] | (rgb[1] << 8) | (rgb[0] << 16);
	return '#' + (0x1000000 + rgb).toString(16).slice(1);
}

/* HSV -> RGB conversion function.
*
* @param hsv Array containing the 3 decimal values: hue, saturation, value, in the [0..1] range
* @return rgb Array containing the 3 integer values: red, green, blue, in the [0..255] range */
function hsv2rgb(hsv)
{
	var H = hsv.hue * 6.0;
	var S = hsv.sat;
	var V = hsv.bright;

	var h = Math.floor(H);
	var f = H - h;
	var p = V * (1.0 - S);
	var q = V * (1.0 - S * f);
	var t = V * (1.0 - S * (1.0 - f));

	var rgb = [];

	switch (h)
	{
		case 0:
		case 6: rgb = [ V, t, p ]; break;
		case 1: rgb = [ q, V, p ]; break;
		case 2: rgb = [ p, V, t ]; break;
		case 3: rgb = [ p, q, V ]; break;
		case 4: rgb = [ t, p, V ]; break;
		case 5: rgb = [ V, p, q ]; break;
		default: rgb = [ 0, 0, 0 ];
	}

	rgb[0] = Math.floor(rgb[0] * 255.0);
	rgb[1] = Math.floor(rgb[1] * 255.0);
	rgb[2] = Math.floor(rgb[2] * 255.0);

	return rgb;
}

/* Converts a string containing an HTML hexadecimal color value to an array of three R/G/B values.
 *
 * @param hex HTML color value (format: #RRGGBB)
 * @return array of the three R/G/B values, in the 0..255 range */
function hex2rgb(hexColor)
{
	// slightly modified version of http://stackoverflow.com/a/11508164
	var h = hexColor.replace('#', ''); // strip the hash at the beginning
	var bigint = null;
	return {r:(((bigint = parseInt(h, 16)) >> 16) & 255), g:(bigint >> 8 & 255), b:(bigint & 255)};
}


/* RGB -> HSV conversion function.
*
* @param rgb Array containing the 3 integer values: red, green, blue, in the [0..255] range
* @return hsv Array containing the 3 decimal values: hue, saturation, value, in the [0..1] range */
function rgb2hsv(rgb)
{
	var r = rgb.r / 255.0;
	var g = rgb.g / 255.0;
	var b = rgb.b / 255.0;

	var minValue = Math.min(r, g, b);
	var maxValue = Math.max(r, g, b);
	var delta = maxValue - minValue;

	var hue = 0;
	var saturation = 0;
	var value = maxValue;

	if (maxValue != 0)
		saturation = delta / maxValue;

	if (delta != 0)
	{

		if (r === maxValue)
			hue = 0 + (g - b) / delta;
		else if (g === maxValue)
			hue = 2 + (b - r) / delta;
		else
			hue = 4 + (r - g) / delta;

		if (hue < 0)
			hue += 6;

		hue = hue / 6.0;
	}

	return {hue:hue, sat:saturation, bright:value};
}



/*Ext.override(Ext.grid.Scroller, {
onAdded: function() {
    this.callParent(arguments);
    var me = this;
    if (me.scrollEl) {
        me.mun(me.scrollEl, 'scroll', me.onElScroll, me);
        me.mon(me.scrollEl, 'scroll', me.onElScroll, me);
    }
}
});*/
/**
 * @class DSS.addon.Framework
 * @extends Ext.container.Viewport Defines an abstract framework for dSS Addons.
 */
Ext.define('DSS.addon.Framework', {

    extend : 'Ext.container.Viewport',

    config : {
        /**
         * The app name, displayed on top of the window Don't forget to localize it
         */
        appName : '',
        /** The app icon displayed next to the app name */
        appIcon : 'default_icon.png',
        /** The app version */
        appVersion : '',
        /** required dSS version, no check done if empty */
        dSSVersion : '',
        /**
         * The appId, used for relative property queries just as in the backend Your relative queries will be prefixed with '/scripts/' + appId + '/'
         */
        appId : ''
    },

    /**
     * Constructor to initialize the class and its dependencies.
     */
    constructor : function(config) {
        var me = this;
        me.initConfig(config);

        me.callParent(arguments);
    },

    // /
    // / Abstract page functions
    // /

    /**
     * Gets the page content
     * 
     * @return {Object} Page content as Ext.container.Container object
     */
    getContent : function() {
        return undefined;
    },

    /**
     * Gets the page's help content
     * 
     * @return {Object} The page's help content as Ext.container.Container object
     */
    getHelp : function() {
        return undefined;
    },

    // /
    // / Initialization
    // /

    /**
     * Initialize the general (static) layout of the page before the rendering is done. Use initPage() to kick off the dynamic parts that require json
     */
    initComponent : function() {
        var me = this;

        document.title = eval('"' + _(me.config.appName) + '"');

        // Attach the default error handlers
        me.on({
            ajaxFailure : me.jsonAjaxFailure
        });

        Ext.apply(me, {
            id : 'app-viewport',
            layout : {
                type : 'border',
                padding : '0 5 5 5' // pad the layout from the window edges
            },
            style : {
                backgroundColor : 'E9FFEE'
            },
            items : [ {
                region : 'center',
                layout : 'border',
                border : false,
                items : [
                        {
                            xtype : 'box',
                            region : 'north',
                            height : 80,
                            border : 0,
                            margins : '0 20 0 20',
                            style : {
                                backgroundColor : 'E9FFEE',                                
                                verticalAlign : 'middle'
                            },
                            html : '<table><tr><td><img id="logo" src="' + me.appIcon + '" alt="app logo" title="' + me.appVersion + '"></td>'
                                    + '<td width="100%"><h1>' +eval('"' + _(me.config.appName) + '"') + '</h1></td>' + '<td><a href="http://www.digitalstrom.com/" alt="'
                                    + _("visit digitalSTROM.org") + '" target="_blank"><img id="logo" src="'+dss.staticDataModel.baseImagePath+'dss/ds_logo.png"'
                                    + 'alt="dS-Logo"></a></td>' + '</tr></table>'
                        }, {
                            id : 'app-content',
                            region : 'center',
                            border : 0,
                            margins : '0 20 0 20',
                            layout : 'fit',
                            items : me.getContent()
                        }, {
                            id : 'app-help-content',
                            region : 'south',
                            title : _('Help'),
                            collapsible : true,
                            collapsed : true,
                            split : true, // enable resizing
                            height : '50%',
                            minHeight : 200,
                            margins : '0 20 20 20',
                            padding : '5 5 0 5',
                            closeAction : 'hide',
                            overflowX : 'hidden',
                            overflowY : 'auto',
                            layout : 'fit',
                            items : me.getHelp()
                        } ]
            } ]
        });
        me.callParent(arguments);
    },

    /**
     * Initialize the dynamic parts of the page after the initial rendering is done
     */
    initPage : function() {
        // Remove the 'loading ...' message from the html
        var start = Ext.get('start');
        if (start) {
            start.remove();
        }
        if (!this.checkdSSVersion(this.dSSVersion, dss.staticDataModel.getdSSVersion(), dss.staticDataModel.getdSSFeed())) {  
            Ext.Msg.show({
                title: _('Error'),
                msg: Ext.String.format2(_('This App needs dSS Version {dssversion} or higher. You need to update your dSS before you can use it'), {dssversion:this.dSSVersion}),
                closable : false,
                buttons: []
             });
        }
    },

    // /
    // / DSS helper functions
    // /

    jsonAjaxFailure : function(err) {
        Ext.Msg.alert(_('Error'), _('Could not send command:') + '\n' + err);
    },

    /**
     * Convert a version string into a number
     * @returns the numeric version
     */
    numericVersion: function(versionString) {
        var sVersion = /\d+\.\d+\.\d+/.exec(versionString);
        var versionSplit = sVersion[0].split('.');
        return parseInt(versionSplit[0], 10)*65536 + parseInt(versionSplit[1], 10)*256 + parseInt(versionSplit[2], 10);
    },

    /**
     * Check if dSSVersion is compatible for fieldtest and production feed. If we are in any other feed
     * , i.e devel, testing,..., the check will return true as we do not check compatibility for these feeds.
     * @param requiredVersion minimal dSS version that is reuired
     * @param currentVersion    current version of dSS, use dss.staticDataModel.getdSSVersion to get it
     */
    checkdSSVersion: function(requiredVersion, installedVersion, installedFeed) {
        try {
	    	if (installedFeed === 'fieldtest' || installedFeed === 'production') {
	            if (this.numericVersion(installedVersion) >= this.numericVersion(requiredVersion)) {
	                return true;
	            } else {
	                return false;    
	            }
	        } else {
	            // we are in any non-production feed i.e. 'devel', 'testing', 'staging-fieldtest', 'exhibition'
	            return true;
	        }
        } catch (e) {
        	// if anything goes wrong ... or NO dSS Version is set, just accept and shut up /** required dSS version, no check done if empty */ 
        	return true; 
        }
    }
});
